! NOTE: This program was incorporated into rscalc_interp_acc.for.
! Note that rdrvaa includes initial conditions in the argument
! list, while rscalc_interp_acc does not.

!----------------- BEGIN RDRVAA -----------------------------
      subroutine rdrvaa(acc,na,omega,damp,dt,rd,rv,aa, d0, v0)
! This is a modified version of "Quake.For", originally
! written by J.M. Roesset in 1971 and modified by
! Stavros A. Anagnostopoulos, Oct. 1986.  The formulation is that of
! Nigam and Jennings (BSSA, v. 59, 909-922, 1969).  

!   acc = acceleration time series
!    na = length of time series
! omega = 2*pi/per
!  damp = fractional damping (e.g., 0.05)
!    dt = time spacing of input
!    rd = relative displacement of oscillator
!    rv = relative velocity of oscillator
!    aa = absolute acceleration of oscillator
! d0,v0 = initial displacement and velocity (usually set to 0.0)

! Dates: 02/11/00 - Modified by David M. Boore, based on RD_CALC
!        03/11/01 - Double precision version
!        03/14/01 - Added d0, v0 (note on 05 March 2010: I recommend 
!                   that they not be used, by setting them to 0.0.  
!                   I've kept them as arguments in the subroutine call
!                   so that I do not have to modify programs that use
!                   this subroutine).                   
!        03/14/01 - Changed name back to rdrvaa
!        01/31/03 - Moved implicit statement before the type declarations
!        10/10/07 - Initial variable assignments and iteration loop modified 
!                   to double-precision (Chris Stephens)
!        03/05/10 - Delete old (single precision) lines of code
!        12/22/10 - Remove minus sign in front of the initialization of y, ydot. 
!                   The minus sign was a remnant of an earlier version where I did not
!                   understand the meaning of y and ydot.
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        05/16/15 - Use Implicit None and replace real*4 and double precision with modern equivalents

      real(4) :: acc(*), omega, damp, dt, rd, rv, aa, d0, v0
      
      real(8) :: d2, bom, d3, omd, om2, c1, omt, omdt, c2, c3, c4, ss, 
     :           cc, bomt, ee, s1, s2, s3, a11, a12, a21, a22,
     :           s4, s5, b11, b12, b21, b22, y, ydot, y1, z, z1, z2, ra
     
      integer :: i, na
      
      
      d2=1.d0-dble(damp)*dble(damp)
      d2=dsqrt(d2)
      bom=dble(damp)*dble(omega)
      d3 = 2.d0*bom                 ! for aa
      omd=dble(omega)*d2
      om2=dble(omega)*dble(omega)
      c1=1.d0/om2
      
      omt=dble(omega)*dble(dt)
      omdt=omd*dble(dt)
      c2=2.d0*dble(damp)/(om2*omt)
      c3=c1+c2
      c4=1.d0/(dble(omega)*omt)
      ss=dsin(omdt)
      cc=dcos(omdt)
      bomt=dble(damp)*omt
      ee=dexp(-bomt)
      ss=ss*ee
      cc=cc*ee
      s1=ss/omd
      s2=s1*bom
      s3=s2+cc
      a11=s3
      a12=s1
      a21=-om2*s1
      a22=cc-s2
      s4=c4*(1.d0-s3)
      s5=s1*c4+c2
      b11=s3*c3-s5
      b12=-c2*s3+s5-c1
      b21=-s1+s4
      b22=-s4
      
      rd=0.
      rv = 0.                           ! for rv
      aa = 0.                           ! for aa
      
      y=    dble(d0)
      ydot= dble(v0)    
!      y=0.
!      ydot=0.

      do i=1, na-1
      
        y1=a11*y+a12*ydot+b11*dble(acc(i))+b12*dble(acc(i+1))
        ydot=a21*y+a22*ydot+b21*dble(acc(i))+b22*dble(acc(i+1))
        y=y1    ! y is the oscillator output at time corresponding to index i
        z=dabs(y)
        if (z > rd) rd=z
        z1 = dabs(ydot)                   ! for rv
        if (z1 > rv) rv = z1            ! for rv
        ra = -d3*ydot -om2*y1            ! for aa
        z2 = dabs(ra)                     ! for aa
        if (z2 > aa) aa = z2            ! for aa
        
      end do
      
      return
      end
!----------------- END RDRVAA -----------------------------
